@@ -1,11 +1,17 @@
 import sys, string, time, copy, gc
 from itertools import *
+from StringIO import StringIO
 
-_TEXT  = "some ASCII text"
-_UTEXT = u"some klingon: \F8D2"
+TREE_FACTOR = 1 # increase tree size with '-l / '-L' cmd option
+
+_TEXT  = "some ASCII text" * TREE_FACTOR
+_UTEXT = u"some klingon: \F8D2" * TREE_FACTOR
 _ATTRIBUTES = {
-    '{attr}test' : _UTEXT,
-    'bla'        : _TEXT
+    '{attr}test1' : _TEXT,
+    '{attr}test2' : _TEXT,
+    'bla1'        : _TEXT,
+    'bla2'        : _TEXT,
+    'bla3'        : _TEXT
     }
 
 def with_attributes(use_attributes):
@@ -43,6 +49,8 @@ def onlylib(*libs):
         return function
     return set_libs
 
+class SkippedTest(Exception):
+    pass
 
 class BenchMarkBase(object):
     atoz = string.ascii_lowercase
@@ -53,6 +61,8 @@ class BenchMarkBase(object):
         'cElementTree' : 'cET'
         }
 
+    SEARCH_TAG = "{cdefg}a00001"
+
     def __init__(self, etree):
         self.etree = etree
         libname = etree.__name__.split('.')[-1]
@@ -64,7 +74,7 @@ class BenchMarkBase(object):
                 setattr(self, fname, lambda : deepcopy(root))
         else:
             def set_property(root, fname):
-                setattr(self, fname, self.et_make_factory(root))
+                setattr(self, fname, self.et_make_clone_factory(root))
 
         attribute_list = list(izip(count(), ({}, _ATTRIBUTES)))
         text_list = list(izip(count(), (None, _TEXT, _UTEXT)))
@@ -87,7 +97,7 @@ class BenchMarkBase(object):
     def tree_builder(self, tree, tn, an):
         return getattr(self, self._tree_builder_name(tree, tn, an))
 
-    def et_make_factory(self, elem):
+    def et_make_clone_factory(self, elem):
         def generate_elem(append, elem, level):
             var = "e" + str(level)
             arg = repr(elem.tag)
@@ -124,61 +134,65 @@ class BenchMarkBase(object):
         return all_trees
 
     def _setup_tree1(self, text, attributes):
-        "tree with 26 2nd level and 520 3rd level children"
+        "tree with 26 2nd level and 520 * TREE_FACTOR 3rd level children"
         atoz = self.atoz
         SubElement = self.etree.SubElement
         current_time = time.time
         t = current_time()
-        root = self.etree.Element('{a}root')
+        root = self.etree.Element('{abc}rootnode')
         for ch1 in atoz:
-            el = SubElement(root, "{b}"+ch1, attributes)
+            el = SubElement(root, "{bcd}"+ch1*5, attributes)
+            el.text = text
             for ch2 in atoz:
-                for i in range(20):
-                    SubElement(el, "{c}%s%03d" % (ch2, i))
+                for i in range(20 * TREE_FACTOR):
+                    SubElement(el, "{cdefg}%s%05d" % (ch2, i))
         t = current_time() - t
         return (root, t)
 
     def _setup_tree2(self, text, attributes):
-        "tree with 520 2nd level and 26 3rd level children"
+        "tree with 520 * TREE_FACTOR 2nd level and 26 3rd level children"
         atoz = self.atoz
         SubElement = self.etree.SubElement
         current_time = time.time
         t = current_time()
-        root = self.etree.Element('{a}root')
+        root = self.etree.Element('{abc}rootnode')
         for ch1 in atoz:
-            for i in range(20):
-                el = SubElement(root, "{b}"+ch1, attributes)
+            for i in range(20 * TREE_FACTOR):
+                el = SubElement(root, "{bcd}"+ch1*5, attributes)
+                el.text = text
                 for ch2 in atoz:
-                    SubElement(el, "{c}%s%03d" % (ch2, i))
+                    SubElement(el, "{cdefg}%s%05d" % (ch2, i))
         t = current_time() - t
         return (root, t)
 
     def _setup_tree3(self, text, attributes):
-        "tree of depth 8 with 3 children per node"
+        "tree of depth 8 + TREE_FACTOR with 3 children per node"
         SubElement = self.etree.SubElement
         current_time = time.time
         t = current_time()
-        root = self.etree.Element('{a}root')
+        root = self.etree.Element('{abc}rootnode')
         children = [root]
-        for i in range(7):
+        for i in range(6 + TREE_FACTOR):
             tag_no = count().next
-            children = [ SubElement(c, "{b}a%d" % i, attributes)
+            children = [ SubElement(c, "{cdefg}a%05d" % i, attributes)
                          for i,c in enumerate(chain(children, children, children)) ]
+        for child in root:
+            child.text = text
         t = current_time() - t
         return (root, t)
 
     def _setup_tree4(self, text, attributes):
         "small tree with 26 2nd level and 2 3rd level children"
-        atoz = self.atoz
         SubElement = self.etree.SubElement
         current_time = time.time
         t = current_time()
-        root = self.etree.Element('{a}root')
+        root = self.etree.Element('{abc}rootnode')
         children = [root]
-        for ch1 in atoz:
-            el = SubElement(root, "{b}"+ch1, attributes)
-            SubElement(el, "{c}a", attributes)
-            SubElement(el, "{c}b", attributes)
+        for ch1 in self.atoz:
+            el = SubElement(root, "{bcd}"+ch1*5, attributes)
+            el.text = text
+            SubElement(el, "{cdefg}a00001", attributes)
+            SubElement(el, "{cdefg}a00002", attributes)
         t = current_time() - t
         return (root, t)
 
@@ -195,8 +209,9 @@ class BenchMarkBase(object):
                 continue
             method = getattr(self, name)
             if hasattr(method, 'LIBS') and self.lib_name not in method.LIBS:
-                benchmarks.append((name, None, (), 0, 0))
-                continue
+                method_call = None
+            else:
+                method_call = method
             if method.__doc__:
                 tree_sets = method.__doc__.split()
             else:
@@ -215,7 +230,7 @@ class BenchMarkBase(object):
             for tree_tuple in tree_tuples:
                 for tn in sorted(getattr(method, 'TEXT', (0,))):
                     for an in sorted(getattr(method, 'ATTRIBUTES', (0,))):
-                        benchmarks.append((name, method, tree_tuple, tn, an))
+                        benchmarks.append((name, method_call, tree_tuple, tn, an))
 
         return benchmarks
 
@@ -248,6 +263,34 @@ class BenchMark(BenchMarkBase):
         for child in reversed(root):
             pass
 
+    @with_attributes(True)
+    @with_attributes(False)
+    @with_text(text=True, utext=True)
+    def bench_tostring_utf8(self, root):
+        self.etree.tostring(root, 'UTF-8')
+
+    @with_attributes(True)
+    @with_attributes(False)
+    @with_text(text=True, utext=True)
+    def bench_tostring_utf16(self, root):
+        self.etree.tostring(root, 'UTF-16')
+
+    @with_attributes(True)
+    @with_attributes(False)
+    @with_text(text=True, utext=True)
+    def bench_tostring_utf8_unicode_XML(self, root):
+        xml = unicode(self.etree.tostring(root, 'UTF-8'), 'UTF-8')
+        self.etree.XML(xml)
+
+    @with_attributes(True)
+    @with_attributes(False)
+    @with_text(text=True, utext=True)
+    def bench_write_utf8_parse_stringIO(self, root):
+        f = StringIO()
+        self.etree.ElementTree(root).write(f, 'UTF-8')
+        f.seek(0)
+        self.etree.parse(f)
+
     def bench_append_from_document(self, root1, root2):
         # == "1,2 2,3 1,3 3,1 3,2 2,1" # trees 1 and 2, or 2 and 3, or ...
         for el in root2:
@@ -300,17 +343,26 @@ class BenchMark(BenchMarkBase):
             child.append(el)
 
     def bench_makeelement(self, root):
-        Element = self.etree.Element
         empty_attrib = {}
         for child in root:
             child.makeelement('{test}test', empty_attrib)
 
-    def bench_replace_children(self, root):
+    def bench_create_elements(self, root):
+        Element = self.etree.Element
+        for child in root:
+            Element('{test}test')
+
+    def bench_replace_children_element(self, root):
         Element = self.etree.Element
         for child in root:
             el = Element('{test}test')
             child[:] = [el]
 
+    def bench_replace_children(self, root):
+        Element = self.etree.Element
+        for child in root:
+            child[:] = [ child[0] ]
+
     def bench_remove_children(self, root):
         for child in root:
             root.remove(child)
@@ -326,9 +378,8 @@ class BenchMark(BenchMarkBase):
     @with_attributes(True)
     def bench_get_attributes(self, root):
         for child in root:
-            child.set('a', 'bla')
-        for child in root:
-            child.get('a')
+            child.get('bla1')
+            child.get('{attr}test1')
 
     def bench_setget_attributes(self, root):
         for child in root:
@@ -374,26 +425,38 @@ class BenchMark(BenchMarkBase):
             for i in repeat:
                 child.text
 
+    @onlylib('lxe')
     def bench_index(self, root):
         for child in root:
             root.index(child)
 
+    @onlylib('lxe')
     def bench_index_slice(self, root):
         for child in root[5:100]:
             root.index(child, 5, 100)
 
+    @onlylib('lxe')
     def bench_index_slice_neg(self, root):
         for child in root[-100:-5]:
             root.index(child, start=-100, stop=-5)
 
-    def bench_getiterator(self, root):
+    def bench_getiterator_all(self, root):
+        list(root.getiterator())
+
+    def bench_getiterator_islice(self, root):
         list(islice(root.getiterator(), 10, 110))
 
     def bench_getiterator_tag(self, root):
-        list(islice(root.getiterator("{b}a"), 3, 10))
+        list(islice(root.getiterator(self.SEARCH_TAG), 3, 10))
 
     def bench_getiterator_tag_all(self, root):
-        list(root.getiterator("{b}a"))
+        list(root.getiterator(self.SEARCH_TAG))
+
+    def bench_findall(self, root):
+        root.findall(".//*")
+
+    def bench_findall_tag(self, root):
+        root.findall(".//" + self.SEARCH_TAG)
 
     @onlylib('lxe')
     def bench_xpath_class(self, root):
@@ -402,53 +465,144 @@ class BenchMark(BenchMarkBase):
             xpath(child)
 
     @onlylib('lxe')
+    def bench_xpath_class_repeat(self, root):
+        for child in root:
+            xpath = self.etree.XPath("./*[0]")
+            xpath(child)
+
+    @onlylib('lxe')
     def bench_xpath_element(self, root):
+        xpath = self.etree.XPathElementEvaluator(root)
         for child in root:
-            xpath = self.etree.XPathElementEvaluator(child)
             xpath.evaluate("./*[0]")
 
+    @onlylib('lxe')
+    def bench_xpath_method(self, root):
+        for child in root:
+            child.xpath("./*[0]")
+
+    @onlylib('lxe')
+    def bench_xpath_extensions_old(self, root):
+        def return_child(_, element):
+            if element:
+                return element[0]
+            else:
+                return ()
+        extensions = {(None, 'child') : return_child}
+        xpath = self.etree.XPath("child(.)", extensions=extensions)
+        for child in root:
+            xpath(child)
+
+    @onlylib('lxe')
+    def bench_xslt_extensions_old(self, root):
+        tree = self.etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:l="test"
+   xmlns:testns="testns"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <l:data>TEST</l:data>
+  <xsl:template match="/">
+    <l:result>
+      <xsl:for-each select="*/*">
+        <xsl:copy-of select="testns:child(.)"/>
+      </xsl:for-each>
+    </l:result>
+  </xsl:template>
+</xsl:stylesheet>
+""")
+        def return_child(_, elements):
+            return elements[0][0]
+
+        extensions = {('testns', 'child') : return_child}
+
+        transform = self.etree.XSLT(tree, extensions)
+        for i in range(10):
+            transform(root)
+
+    @onlylib('lxe')
+    def bench_xslt_document(self, root):
+        transform = self.etree.XSLT(self.etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:l="test"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <l:data>TEST</l:data>
+  <xsl:template match="/">
+    <l:result>
+      <xsl:for-each select="*/*">
+        <l:test><xsl:copy-of select="document('')//l:data/text()"/></l:test>
+      </xsl:for-each>
+    </l:result>
+  </xsl:template>
+</xsl:stylesheet>
+"""))
+        transform(root)
+
+
 ############################################################
 # Main program
 ############################################################
 
 if __name__ == '__main__':
     import_lxml = True
+    callgrind_zero = False
     if len(sys.argv) > 1:
         try:
             sys.argv.remove('-i')
+            # run benchmark 'inplace'
             sys.path.insert(0, 'src')
         except ValueError:
             pass
 
         try:
             sys.argv.remove('-nolxml')
+            # run without lxml
             import_lxml = False
         except ValueError:
             pass
 
+        try:
+            sys.argv.remove('-z')
+            # reset callgrind after tree setup
+            callgrind_zero = True
+        except ValueError:
+            pass
+
+        try:
+            sys.argv.remove('-l')
+            # use large trees
+            TREE_FACTOR *= 2
+        except ValueError:
+            pass
+
+        try:
+            sys.argv.remove('-L')
+            # use LARGE trees
+            TREE_FACTOR *= 2
+        except ValueError:
+            pass
+
     _etrees = []
     if import_lxml:
         from lxml import etree
         _etrees.append(etree)
 
     if len(sys.argv) > 1:
-        try:
-            sys.argv.remove('-a')
-        except ValueError:
-            pass
-        else:
-            try:
-                from elementtree import ElementTree as ET
-                _etrees.append(ET)
-            except ImportError:
-                pass
-
+        if '-a' in sys.argv or '-c' in sys.argv:
+            # 'all' or 'C-implementations' ?
             try:
                 import cElementTree as cET
                 _etrees.append(cET)
             except ImportError:
                 pass
 
+        try:
+            # 'all' ?
+            sys.argv.remove('-a')
+            from elementtree import ElementTree as ET
+            _etrees.append(ET)
+        except (ValueError, ImportError):
+            pass
+
     if not _etrees:
         print "No library to test. Exiting."
         sys.exit(1)
@@ -463,14 +617,17 @@ if __name__ == '__main__':
     if len(sys.argv) > 1:
         selected = []
         for name in sys.argv[1:]:
-            if not name.startswith('bench_'):
-                name = 'bench_' + name
             selected.append(name)
-        benchmarks = [ [ b for b in bs if b[0] in selected ]
+        benchmarks = [ [ b for b in bs
+                         if [ match for match in selected
+                              if match in b[0] ] ]
                        for bs in benchmarks ]
 
     import time
     def run_bench(suite, method_name, method_call, tree_set, tn, an):
+        if method_call is None:
+            raise SkippedTest
+
         current_time = time.time
         call_repeat = range(10)
 
@@ -515,23 +672,33 @@ if __name__ == '__main__':
             print "     T%d:" % (i+1), ' '.join("%6.4f" % t for t in tree_times)
     print
 
+    if callgrind_zero:
+        cmd = open("callgrind.cmd", 'w')
+        cmd.write('Zero\n')
+        cmd.close()
+
     for bench_calls in izip(*benchmarks):
         for lib, (bench, benchmark_setup) in enumerate(izip(benchmark_suites, bench_calls)):
-            bench_name, method_call = benchmark_setup[:2]
+            bench_name = benchmark_setup[0]
             tree_set_name = build_treeset_name(*benchmark_setup[-3:])
-            print "%-3s: %-23s" % (bench.lib_name, bench_name[6:29]),
-            if method_call is None:
-                print "skipped"
-                continue
-
+            print "%-3s: %-28s" % (bench.lib_name, bench_name[6:34]),
             print "(%-10s)" % tree_set_name,
             sys.stdout.flush()
 
-            result = run_bench(bench, *benchmark_setup)
-
-            for t in result:
-                print "%9.4f" % t,
-            print "msec/pass, best: %9.4f" % min(result)
+            try:
+                result = run_bench(bench, *benchmark_setup)
+            except SkippedTest:
+                print "skipped"
+            except KeyboardInterrupt:
+                print "interrupted by user"
+                sys.exit(1)
+            except Exception, e:
+                print "failed: %s: %s" % (e.__class__.__name__, e)
+            else:
+                print "%9.4f msec/pass, best of (" % min(result),
+                for t in result:
+                    print "%9.4f" % t,
+                print ")"
 
         if len(benchmark_suites) > 1:
             print # empty line between different benchmarks
