@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 
 """
 Tests for the ElementTree API
@@ -9,7 +9,7 @@ for IO related test cases.
 """
 
 import unittest, doctest
-import os, shutil, tempfile, copy
+import os, re, shutil, tempfile, copy
 
 from common_imports import StringIO, etree, ElementTree, HelperTestCase, fileInTestDir, canonicalize
 
@@ -89,6 +89,24 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('two', root[1].tag)
         self.assertEquals('three', root[2].tag)
         
+    def test_element_contains(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        root1 = Element('root')
+        SubElement(root1, 'one')
+        self.assert_(root1[0] in root1)
+
+        root2 = Element('root')
+        SubElement(root2, 'two')
+        SubElement(root2, 'three')
+        self.assert_(root2[0] in root2)
+        self.assert_(root2[1] in root2)
+
+        self.assertFalse(root1[0] in root2)
+        self.assertFalse(root2[0] in root1)
+        self.assertFalse(None in root2)
+
     def test_element_indexing_with_text(self):
         ElementTree = self.etree.ElementTree
         
@@ -169,6 +187,31 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals(None, root.text)
         self.assertEquals('One', root[0].text)
 
+    def test_text_escape_in(self):
+        ElementTree = self.etree.ElementTree
+
+        f = StringIO('<doc>This is &gt; than a text</doc>')
+        doc = ElementTree(file=f)
+        root = doc.getroot()
+        self.assertEquals('This is > than a text', root.text)
+
+    def test_text_escape_out(self):
+        Element = self.etree.Element
+
+        a = Element("a")
+        a.text = "<>&"
+        self.assertXML('<a>&lt;&gt;&amp;</a>',
+                       a)
+
+    def test_text_escape_tostring(self):
+        tostring = self.etree.tostring
+        Element  = self.etree.Element
+
+        a = Element("a")
+        a.text = "<>&"
+        self.assertEquals('<a>&lt;&gt;&amp;</a>',
+                         tostring(a))
+
     def test_tail(self):
         ElementTree = self.etree.ElementTree
         
@@ -340,6 +383,7 @@ class ETreeTestCaseBase(unittest.TestCase):
           <h1 id="chapter1">...</h1>
           <p id="note1" class="note">...</p>
           <p>Regular paragraph.</p>
+          <p xml:id="xmlid">XML:ID paragraph.</p>
           <p id="warn1" class="warning">...</p>
         </document>
         '''
@@ -351,7 +395,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         expected = {
             "chapter1" : root[0],
             "note1"    : root[1],
-            "warn1"    : root[3]
+            "warn1"    : root[4]
             }
         self.assertEquals(dic, expected)
 
@@ -475,6 +519,30 @@ class ETreeTestCaseBase(unittest.TestCase):
         result.sort()
         self.assertEquals(['alpha', 'beta', 'gamma'], result)
 
+    def test_findall(self):
+        XML = self.etree.XML
+        root = XML('<a><b><c/></b><b/><c><b/></c></a>')
+        self.assertEquals(len(root.findall("c")), 1)
+        self.assertEquals(len(root.findall(".//c")), 2)
+        self.assertEquals(len(root.findall(".//b")), 3)
+        self.assertEquals(len(root.findall(".//b")[0]), 1)
+        self.assertEquals(len(root.findall(".//b")[1]), 0)
+        self.assertEquals(len(root.findall(".//b")[2]), 0)
+
+    def test_findall_ns(self):
+        XML = self.etree.XML
+        root = XML('<a xmlns:x="X" xmlns:y="Y"><x:b><c/></x:b><b/><c><x:b/><b/></c><b/></a>')
+        self.assertEquals(len(root.findall(".//{X}b")), 2)
+        self.assertEquals(len(root.findall(".//b")), 3)
+        self.assertEquals(len(root.findall("b")), 2)
+
+    def test_element_with_attributes_keywords(self):
+        Element = self.etree.Element
+        
+        el = Element('tag', foo='Foo', bar='Bar')
+        self.assertEquals('Foo', el.attrib['foo'])
+        self.assertEquals('Bar', el.attrib['bar'])
+
     def test_element_with_attributes(self):
         Element = self.etree.Element
         
@@ -482,13 +550,30 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('Foo', el.attrib['foo'])
         self.assertEquals('Bar', el.attrib['bar'])
 
+    def test_element_with_attributes_ns(self):
+        Element = self.etree.Element
+
+        el = Element('tag', {'{ns1}foo':'Foo', '{ns2}bar':'Bar'})
+        self.assertEquals('Foo', el.attrib['{ns1}foo'])
+        self.assertEquals('Bar', el.attrib['{ns2}bar'])
+
     def test_subelement_with_attributes(self):
         Element =  self.etree.Element
         SubElement = self.etree.SubElement
         
         el = Element('tag')
-        SubElement(el, 'foo', baz="Baz")
+        SubElement(el, 'foo', attrib={'foo':'Foo'}, baz="Baz")
         self.assertEquals("Baz", el[0].attrib['baz'])
+        self.assertEquals('Foo', el[0].attrib['foo'])
+
+    def test_subelement_with_attributes_ns(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        el = Element('tag')
+        SubElement(el, 'foo', {'{ns1}foo':'Foo', '{ns2}bar':'Bar'})
+        self.assertEquals('Foo', el[0].attrib['{ns1}foo'])
+        self.assertEquals('Bar', el[0].attrib['{ns2}bar'])
         
     def test_write(self):
         ElementTree = self.etree.ElementTree
@@ -646,9 +731,19 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         a = Element('a')
         a.append(Comment('foo'))
-        self.assertXML(
-            '<a><!-- foo --></a>',
-            a)
+        self.assertEquals(a[0].text, 'foo')
+
+    def test_comment_text(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        Comment = self.etree.Comment
+
+        a = Element('a')
+        a.append(Comment('foo'))
+        self.assertEquals(a[0].text, 'foo')
+
+        a[0].text = "TEST"
+        self.assertEquals(a[0].text, 'TEST')
         
     def test_comment_whitespace(self):
         Element = self.etree.Element
@@ -657,9 +752,7 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         a = Element('a')
         a.append(Comment(' foo  '))
-        self.assertXML(
-            '<a><!--  foo   --></a>',
-            a)
+        self.assertEquals(a[0].text, ' foo  ')
         
     def test_comment_nonsense(self):
         Comment = self.etree.Comment
@@ -706,7 +799,17 @@ class ETreeTestCaseBase(unittest.TestCase):
             a)
         self.assertXML('<c></c>',
                        c)
-        
+
+    def test_setitem_replace(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        SubElement(a, 'b')
+        d = Element('d')
+        a[0] = d
+        self.assertXML('<a><d></d></a>', a)
+
     def test_setitem_indexerror(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -730,7 +833,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertXML(
             '<a><c></c>C2</a>',
             a)
-        
+
     def test_tag_write(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -748,6 +851,43 @@ class ETreeTestCaseBase(unittest.TestCase):
             '<c><b></b></c>',
             a)
 
+    def test_tag_reset_ns(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        tostring = self.etree.tostring
+
+        a = Element('{a}a')
+        b1 = SubElement(a, '{a}b')
+        b2 = SubElement(a, '{b}b')
+
+        self.assertEquals('{a}b',  b1.tag)
+
+        b1.tag = 'c'
+
+        # can't use C14N here!
+        self.assertEquals('c', b1.tag)
+        self.assertEquals('<c', tostring(b1)[:2])
+        self.assert_('<c' in tostring(a))
+
+    def test_tag_reset_root_ns(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        tostring = self.etree.tostring
+
+        a = Element('{a}a')
+        b1 = SubElement(a, '{a}b')
+        b2 = SubElement(a, '{b}b')
+
+        a.tag = 'c'
+
+        self.assertEquals(
+            'c',
+            a.tag)
+
+        # can't use C14N here!
+        self.assertEquals('c',  a.tag)
+        self.assertEquals('<c', tostring(a)[:2])
+
     def test_delitem(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -1082,7 +1222,7 @@ class ETreeTestCaseBase(unittest.TestCase):
             list(a.getiterator('a')))
         self.assertEquals(
             [a2],
-            list(e.getiterator('a'))) 
+            list(c.getiterator('a')))
 
     def test_getiterator_with_text(self):
         Element = self.etree.Element
@@ -1533,7 +1673,28 @@ class ETreeTestCaseBase(unittest.TestCase):
         # as namespace is not moved along with it
         del one
         self.assertEquals('{http://a.b.c}baz', baz.tag)
-        
+
+    def test_attribute_xmlns_move(self):
+        Element = self.etree.Element
+
+        root = Element('element')
+
+        subelement = Element('subelement',
+                             {"{http://www.w3.org/XML/1998/namespace}id": "foo"})
+        self.assertEquals(1, len(subelement.attrib))
+        self.assertEquals(
+            "foo",
+            subelement.get("{http://www.w3.org/XML/1998/namespace}id"))
+
+        root.append(subelement)
+        self.assertEquals(1, len(subelement.attrib))
+        self.assertEquals(
+            {"{http://www.w3.org/XML/1998/namespace}id" : "foo"}.items(),
+            subelement.attrib.items())
+        self.assertEquals(
+            "foo",
+            subelement.get("{http://www.w3.org/XML/1998/namespace}id"))
+
     def test_tostring(self):
         tostring = self.etree.tostring
         Element = self.etree.Element
@@ -1559,10 +1720,6 @@ class ETreeTestCaseBase(unittest.TestCase):
                           canonicalize(tostring(b)))
         self.assertEquals('<c><d></d></c>',
                           canonicalize(tostring(c)))
-
-    def test_tostring_none(self):
-        tostring = self.etree.tostring
-        self.assertRaises(AssertionError, self.etree.tostring, None)
         
     def test_tostring_element_tail(self):
         tostring = self.etree.tostring
@@ -1589,7 +1746,7 @@ class ETreeTestCaseBase(unittest.TestCase):
     def test_parse_file_nonexistent(self):
         parse = self.etree.parse
         self.assertRaises(IOError, parse, fileInTestDir('notthere.xml'))  
-        
+
     def test_parse_file_object(self):
         parse = self.etree.parse
         # from file object
@@ -1610,28 +1767,24 @@ class ETreeTestCaseBase(unittest.TestCase):
             '<a><b></b></a>',
             tree.getroot()
            )
-        
+
+    def test_parse_with_encoding(self):
+        # this can fail in libxml2 <= 2.6.22
+        parse = self.etree.parse
+        tree = parse(StringIO('<?xml version="1.0" encoding="ascii"?><html/>'))
+        self.assertXML('<html></html>',
+                       tree.getroot())
+
     def test_encoding(self):
-        ElementTree = self.etree.ElementTree
         Element = self.etree.Element
 
         a = Element('a')
         a.text = u'Søk på nettet'
         self.assertXML(
             u'<a>Søk på nettet</a>'.encode('UTF-8'),
-            a)
+            a, 'utf-8')
 
-    def test_encoding2(self):
-        ElementTree = self.etree.ElementTree
-        Element = self.etree.Element
-
-        a = Element('a')
-        a.text = u'Søk på nettet'
-        self.assertXML(
-            u'<a>Søk på nettet</a>'.encode('UTF-8'),
-            a, 'UTF-8')
-        
-    def test_encoding3(self):
+    def test_encoding_exact(self):
         ElementTree = self.etree.ElementTree
         Element = self.etree.Element
 
@@ -1640,25 +1793,36 @@ class ETreeTestCaseBase(unittest.TestCase):
         
         f = StringIO()
         tree = ElementTree(element=a)
-        tree.write(f, 'UTF-8')
-        data = f.getvalue()
+        tree.write(f, 'utf-8')
+        self.assertEquals(u'<a>Søk på nettet</a>'.encode('UTF-8'),
+                          f.getvalue())
 
-        # XXX prologue generation seems to be inconsistent between libraries..
-        xml = u'<a>Søk på nettet</a>'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
-        self.assert_(data in [xml, prologue + xml])
+    def test_encoding_latin1(self):
+        ElementTree = self.etree.ElementTree
+        Element = self.etree.Element
 
-##     # ignore wrong (left-over?) encoding declaration in unicode strings
-##     def _test_wrong_unicode_encoding(self):
-##         XML = self.etree.XML
+        a = Element('a')
+        a.text = u'Søk på nettet'
 
-##         test_utf = u'<?xml version=\'1.0\' encoding=\'iso-8859-1\'?><a>Søk på nettet</a>'
-##         parsed = XML(test_utf)
-##         self.assertXML(
-##             u'<a>Søk på nettet</a>'.encode('UTF-8'),
-##             parsed, 'UTF-8')
-        
-    def test_encoding_default_encoding(self):
+        f = StringIO()
+        tree = ElementTree(element=a)
+        tree.write(f, 'iso-8859-1')
+        result = f.getvalue()
+        declaration = "<?xml version=\'1.0\' encoding=\'iso-8859-1\'?>"
+        self.assertEncodingDeclaration(result,'iso-8859-1')
+        result = result.split('?>', 1)[-1]
+        if result[0] == '\n':
+            result = result[1:]
+        self.assertEquals(u'<a>Søk på nettet</a>'.encode('iso-8859-1'),
+                          result)
+
+    # raise error on wrong (left-over?) encoding declaration in unicode strings
+    def _test_wrong_unicode_encoding(self):
+        XML = self.etree.XML
+        test_utf = u'<?xml version=\'1.0\' encoding=\'iso-8859-1\'?><a>Søk på nettet</a>'
+        self.assertRaises(SyntaxError, XML, test_utf)
+
+    def test_encoding_write_default_encoding(self):
         ElementTree = self.etree.ElementTree
         Element = self.etree.Element
 
@@ -1670,21 +1834,26 @@ class ETreeTestCaseBase(unittest.TestCase):
         tree.write(f)
         data = f.getvalue()
         self.assertEquals(
-            '<a>S&#248;k p&#229; nettet</a>',
+            u'<a>Søk på nettet</a>'.encode('ASCII', 'xmlcharrefreplace'),
             data)
 
     def test_encoding_tostring(self):
         Element = self.etree.Element
         tostring = self.etree.tostring
 
-        # XXX prologue generation seems to be inconsistent between libraries..
-        xml = u'<a>Søk på nettet</a>'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
+        a = Element('a')
+        a.text = u'Søk på nettet'
+        self.assertEquals(u'<a>Søk på nettet</a>'.encode('UTF-8'),
+                         tostring(a, 'utf-8'))
+
+    def test_encoding_tostring_unknown(self):
+        Element = self.etree.Element
+        tostring = self.etree.tostring
         
         a = Element('a')
         a.text = u'Søk på nettet'
-        self.assert_(tostring(a, 'UTF-8') in [xml, prologue + xml])
-        
+        self.assertRaises(LookupError, tostring, a, 'Invalid Encoding')
+
     def test_encoding_tostring_sub(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -1693,11 +1862,8 @@ class ETreeTestCaseBase(unittest.TestCase):
         a = Element('a')
         b = SubElement(a, 'b')
         b.text = u'Søk på nettet'
-
-        # XXX prologue generation seems to be inconsistent between libraries..
-        xml = u'<b>Søk på nettet</b>'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
-        self.assert_(tostring(b, 'UTF-8') in [xml, prologue + xml])
+        self.assertEquals(u'<b>Søk på nettet</b>'.encode('UTF-8'),
+                         tostring(b, 'utf-8'))
 
     def test_encoding_tostring_sub_tail(self):
         Element = self.etree.Element
@@ -1708,9 +1874,8 @@ class ETreeTestCaseBase(unittest.TestCase):
         b = SubElement(a, 'b')
         b.text = u'Søk på nettet'
         b.tail = u'Søk'
-        xml = u'<b>Søk på nettet</b>Søk'.encode('UTF-8')
-        prologue = u'<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n'.encode('UTF-8')
-        self.assert_(tostring(b, 'UTF-8') in [xml, prologue + xml])
+        self.assertEquals(u'<b>Søk på nettet</b>Søk'.encode('UTF-8'),
+                         tostring(b, 'utf-8'))
         
     def test_encoding_tostring_default_encoding(self):
         Element = self.etree.Element
@@ -1734,10 +1899,10 @@ class ETreeTestCaseBase(unittest.TestCase):
         b = SubElement(a, 'b')
         b.text = u'Søk på nettet'
 
-        # the same, just hex versus decimal
         expected = '<b>S&#248;k p&#229; nettet</b>'
-        expected2 = '<b>S&#xF8;k p&#xE5; nettet</b>'
-        self.assert_(tostring(b) in [expected, expected2])
+        self.assertEquals(
+            expected,
+            tostring(b))
 
     def test_deepcopy(self):
         Element = self.etree.Element
@@ -1766,6 +1931,21 @@ class ETreeTestCaseBase(unittest.TestCase):
         self.assertEquals('Foo', a.text)
         # XXX ElementTree will share nodes, but lxml.etree won't..
         
+    def test_deepcopy_append(self):
+        # previously caused a crash
+        Element = self.etree.Element
+        tostring = self.etree.tostring
+        
+        a = Element('a')
+        b = copy.deepcopy(a)
+        a.append( Element('C') )
+        b.append( Element('X') )
+
+        self.assertEquals('<a><C/></a>',
+                          tostring(a).replace(' ', ''))
+        self.assertEquals('<a><X/></a>',
+                          tostring(b).replace(' ', ''))
+
     def test_element_boolean(self):
         etree = self.etree
         e = etree.Element('foo')
@@ -1815,7 +1995,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         f = StringIO()
         tree = ElementTree(element=element)
         tree.write(f, encoding)
-        data = f.getvalue()
+        data = unicode(f.getvalue(), encoding)
         return canonicalize(data)
 
     def _writeElementFile(self, element, encoding='us-ascii'):
@@ -1823,15 +2003,17 @@ class ETreeTestCaseBase(unittest.TestCase):
         """
         ElementTree = self.etree.ElementTree
         handle, filename = tempfile.mkstemp()
-        f = open(filename, 'wb')
-        tree = ElementTree(element=element)
-        tree.write(f, encoding)
-        f.close()
-        f = open(filename, 'rb')
-        data = f.read()
-        f.close()
-        os.close(handle)
-        os.remove(filename)
+        try:
+            f = open(filename, 'wb')
+            tree = ElementTree(element=element)
+            tree.write(f, encoding)
+            f.close()
+            f = open(filename, 'rb')
+            data = unicode(f.read(), encoding)
+            f.close()
+        finally:
+            os.close(handle)
+            os.remove(filename)
         return canonicalize(data)
 
     def assertXML(self, expected, element, encoding='us-ascii'):
@@ -1841,6 +2023,13 @@ class ETreeTestCaseBase(unittest.TestCase):
         """
         self.assertEquals(expected, self._writeElement(element, encoding))
         self.assertEquals(expected, self._writeElementFile(element, encoding))
+
+    def assertEncodingDeclaration(self, result, encoding):
+        "Checks if the result XML byte string specifies the encoding."
+        has_encoding = re.compile(r"<\?xml[^>]+ encoding=[\"']([^\"']+)[\"']").match
+        self.assert_(has_encoding(result))
+        result_encoding = has_encoding(result).group(1)
+        self.assertEquals(result_encoding.upper(), encoding.upper())
         
     def _rootstring(self, tree):
         return self.etree.tostring(tree.getroot()).replace(' ', '').replace('\n', '')
@@ -1890,8 +2079,6 @@ def test_suite():
     suite.addTests([unittest.makeSuite(ETreeTestCase)])
     if ElementTree:
         suite.addTests([unittest.makeSuite(ElementTreeTestCase)])
-    suite.addTests(
-        [doctest.DocFileSuite('../../../doc/api.txt')])
     return suite
 
 if __name__ == '__main__':
