@@ -1,12 +1,13 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 import unittest, doctest
 
-from StringIO import StringIO
-from lxml import etree
+from common_imports import StringIO, etree, SillyFileLike
 
 ascii_uni = u'a'
 
-uni = u'á\uF8D2' # klingon etc.
+uni = u'Ã¡\uF8D2' # klingon etc.
+
+uxml = u"<test><title>test Ã¡\uF8D2</title><h1>page Ã¡\uF8D2 title</h1></test>"
 
 class UnicodeTestCase(unittest.TestCase):
     def test_unicode_xml(self):
@@ -34,12 +35,19 @@ class UnicodeTestCase(unittest.TestCase):
 
     def test_unicode_comment(self):
         el = etree.Comment(uni)
-        self.assertEquals(' %s ' % uni, el.text)
+        self.assertEquals(uni, el.text)
 
     def test_unicode_parse_stringio(self):
         el = etree.parse(StringIO(u'<p>%s</p>' % uni)).getroot()
         self.assertEquals(uni, el.text)
 
+    def test_parse_fileobject_unicode(self):
+        # parse unicode from unamed file object (not support by ElementTree)
+        f = SillyFileLike(uxml)
+        root = etree.parse(f).getroot()
+        self.assertEquals(unicode(etree.tostring(root, 'UTF-8'), 'UTF-8'),
+                          uxml)
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(UnicodeTestCase)])
