@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# -*- coding: utf-8 -*-
 
 """
 Test cases related to XSLT processing
@@ -6,7 +6,7 @@ Test cases related to XSLT processing
 
 import unittest, doctest
 
-from common_imports import etree, HelperTestCase, fileInTestDir
+from common_imports import etree, StringIO, HelperTestCase, fileInTestDir
 
 class ETreeXSLTTestCase(HelperTestCase):
     """XPath tests etree"""
@@ -30,6 +30,9 @@ class ETreeXSLTTestCase(HelperTestCase):
 ''',
                           st.tostring(res))
 
+    def test_xslt_elementtree_error(self):
+        self.assertRaises(ValueError, etree.XSLT, etree.ElementTree())
+
     def test_xslt_utf8(self):
         tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
         style = self.parse('''\
@@ -70,6 +73,78 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals(expected,
                           unicode(str(res), 'UTF-16'))
 
+    def test_xslt_encoding_override(self):
+        tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:output encoding="UTF-8"/>
+  <xsl:template match="/">
+    <foo><xsl:value-of select="/a/b/text()" /></foo>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = st.apply(tree)
+        expected = u"""\
+<?xml version='1.0' encoding='UTF-16'?>
+<foo>\uF8D2</foo>"""
+
+        f = StringIO()
+        res.write(f, 'UTF-16')
+        result = unicode(f.getvalue(), 'UTF-16')
+        self.assertEquals(expected,
+                          result)
+
+    def test_xslt_unicode(self):
+        tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:output encoding="UTF-16"/>
+  <xsl:template match="/">
+    <foo><xsl:value-of select="/a/b/text()" /></foo>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = st.apply(tree)
+        expected = u'''\
+<?xml version="1.0"?>
+<foo>\uF8D2</foo>
+'''
+        self.assertEquals(expected,
+                          unicode(res))
+
+    def test_exslt(self):
+        tree = self.parse('<a><b>B</b><c>C</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:math="http://exslt.org/math"
+    xmlns:str="http://exslt.org/strings"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
+    exclude-result-prefixes="math str xsl">
+  <xsl:template match="text()">
+    <xsl:value-of select="str:align(string(.), '---', 'center')" />
+  </xsl:template>
+  <xsl:template match="*">
+    <xsl:copy>
+      <xsl:attribute name="pi">
+        <xsl:value-of select="math:constant('PI', count(*)+2)"/>
+      </xsl:attribute>
+      <xsl:apply-templates/>
+    </xsl:copy>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = st(tree)
+        self.assertEquals('''\
+<?xml version="1.0"?>
+<a pi="3.14"><b pi="3">-B-</b><c pi="3">-C-</c></a>
+''',
+                          st.tostring(res))
+
     def test_xslt_input(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
         style = self.parse('''\
@@ -86,7 +161,6 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertRaises(TypeError, etree.XSLT, None)
 
     def test_xslt_input_partial_doc(self):
-        tree = self.parse('<a><b>B</b><c>C</c></a>')
         style = self.parse('''\
 <otherroot>
 <xsl:stylesheet version="1.0"
@@ -106,10 +180,10 @@ class ETreeXSLTTestCase(HelperTestCase):
     def test_xslt_broken(self):
         tree = self.parse('<a/>')
         style = self.parse('''\
-<xslt:stylesheet version="1.0"
+<xsl:stylesheet version="1.0"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
     <xsl:foo />
-</xslt:stylesheet>''')
+</xsl:stylesheet>''')
         self.assertRaises(etree.XSLTParseError,
                           etree.XSLT, style)
 
@@ -262,10 +336,11 @@ class ETreeXSLTTestCase(HelperTestCase):
         etree.tostring(result.getroot())
         result = transform.apply(source)
         etree.tostring(result.getroot())
-
-        result = transform(source)
-        result = transform(source)
         str(result)
+
+        result1 = transform(source)
+        result2 = transform(source)
+        self.assertEquals(str(result1), str(result2))
         result = transform(source)
         str(result)
 
@@ -283,6 +358,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         style = etree.XSLT(styledoc)
         result = style.apply(source)
         self.assertEqual('', style.tostring(result))
+        self.assertEqual('', str(result))
 
     def test_xslt_shortcut(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
@@ -352,7 +428,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         def mytext(ctxt, values):
             return 'X' * len(values)
 
-        result = tree.xslt(style, {'testns' : {'mytext' : mytext}})
+        result = tree.xslt(style, {('testns', 'mytext') : mytext})
         self.assertEquals(self._rootstring(result),
                           '<A>X</A>')
 
@@ -376,6 +452,27 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals(self._rootstring(result),
                           '<A>X</A>')
 
+    def test_xslt_document_XML(self):
+        # make sure document('') works from parsed strings
+        xslt = etree.XSLT(etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="/">
+    <test>TEXT<xsl:copy-of select="document('')//test"/></test>
+  </xsl:template>
+</xsl:stylesheet>
+"""))
+        result = xslt(etree.XML('<a/>'))
+        root = result.getroot()
+        self.assertEquals(root.tag,
+                          'test')
+        self.assertEquals(root[0].tag,
+                          'test')
+        self.assertEquals(root[0].text,
+                          'TEXT')
+        self.assertEquals(root[0][0].tag,
+                          '{http://www.w3.org/1999/XSL/Transform}copy-of')
+
     def test_xslt_document_parse(self):
         # make sure document('') works from loaded files
         xslt = etree.XSLT(etree.parse(fileInTestDir("test-document.xslt")))
@@ -396,6 +493,96 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals(root[0].tag,
                           '{http://www.w3.org/1999/XSL/Transform}stylesheet')
 
+    def test_xslt_document_error(self):
+        # make sure document('') works from parsed strings
+        xslt = etree.XSLT(etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="/">
+    <test>TEXT<xsl:copy-of select="document('uri:__junkfood__is__evil__')//test"/></test>
+  </xsl:template>
+</xsl:stylesheet>
+"""))
+        self.assertRaises(etree.XSLTApplyError, xslt, etree.XML('<a/>'))
+
+    def test_exslt_regexp_test(self):
+        xslt = etree.XSLT(etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:regexp="http://exslt.org/regular-expressions"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="*">
+    <test><xsl:copy-of select="*[regexp:test(string(.), '8.')]"/></test>
+  </xsl:template>
+</xsl:stylesheet>
+"""))
+        result = xslt(etree.XML('<a><b>123</b><b>098</b><b>987</b></a>'))
+        root = result.getroot()
+        self.assertEquals(root.tag,
+                          'test')
+        self.assertEquals(len(root), 1)
+        self.assertEquals(root[0].tag,
+                          'b')
+        self.assertEquals(root[0].text,
+                          '987')
+
+    def test_exslt_regexp_replace(self):
+        xslt = etree.XSLT(etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:regexp="http://exslt.org/regular-expressions"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="*">
+    <test>
+      <xsl:copy-of select="regexp:replace(string(.), 'd.', '',   'XX')"/>
+      <xsl:text>-</xsl:text>
+      <xsl:copy-of select="regexp:replace(string(.), 'd.', 'gi', 'XX')"/>
+    </test>
+  </xsl:template>
+</xsl:stylesheet>
+"""))
+        result = xslt(etree.XML('<a>abdCdEeDed</a>'))
+        root = result.getroot()
+        self.assertEquals(root.tag,
+                          'test')
+        self.assertEquals(len(root), 0)
+        self.assertEquals(root.text, 'abXXdEeDed-abXXXXeXXd')
+
+    def test_exslt_regexp_match(self):
+        xslt = etree.XSLT(etree.XML("""\
+<xsl:stylesheet version="1.0"
+   xmlns:regexp="http://exslt.org/regular-expressions"
+   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="*">
+    <test>
+      <test1><xsl:copy-of  select="regexp:match(string(.), 'd.')"/></test1>
+      <test2><xsl:copy-of  select="regexp:match(string(.), 'd.', 'g')"/></test2>
+      <test2i><xsl:copy-of select="regexp:match(string(.), 'd.', 'gi')"/></test2i>
+    </test>
+  </xsl:template>
+</xsl:stylesheet>
+"""))
+        result = xslt(etree.XML('<a>abdCdEeDed</a>'))
+        root = result.getroot()
+        self.assertEquals(root.tag,  'test')
+        self.assertEquals(len(root), 3)
+
+        self.assertEquals(len(root[0]), 1)
+        self.assertEquals(root[0][0].tag, 'match')
+        self.assertEquals(root[0][0].text, 'dC')
+
+        self.assertEquals(len(root[1]), 2)
+        self.assertEquals(root[1][0].tag, 'match')
+        self.assertEquals(root[1][0].text, 'dC')
+        self.assertEquals(root[1][1].tag, 'match')
+        self.assertEquals(root[1][1].text, 'dE')
+
+        self.assertEquals(len(root[2]), 3)
+        self.assertEquals(root[2][0].tag, 'match')
+        self.assertEquals(root[2][0].text, 'dC')
+        self.assertEquals(root[2][1].tag, 'match')
+        self.assertEquals(root[2][1].text, 'dE')
+        self.assertEquals(root[2][2].tag, 'match')
+        self.assertEquals(root[2][2].text, 'De')
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ETreeXSLTTestCase)])
